from rich.console import Console
from rich.table import Column, Table
from importlib.metadata import version, PackageNotFoundError,distributions  # Python 3.8+
import requests
ml_dl_libraries = {
    "numpy": {
        "url": "https://numpy.org/",
        "description": "用于数值计算",
        "category": "数据处理与可视化"
    },
    "pandas": {
        "url": "https://pandas.pydata.org/",
        "description": "数据处理与操作",
        "category": "数据处理与可视化"
    },
    "scikit-learn": {
        "url": "https://scikit-learn.org/",
        "description": "scikit-learn 经典的机器学习库",
        "category": "机器学习"
    },
    "tensorflow": {
        "url": "https://www.tensorflow.org/",
        "description": "Google 的深度学习框架",
        "category": "深度学习"
    },
    "keras": {
        "url": "https://keras.io/",
        "description": "基于 TensorFlow 的高级深度学习 API",
        "category": "深度学习"
    },
    "pytorch": {
        "url": "https://pytorch.org/",
        "description": "Facebook 开发的深度学习框架",
        "category": "深度学习"
    },
    "xgboost": {
        "url": "https://xgboost.readthedocs.io/",
        "description": "高效的梯度提升库，常用于比赛",
        "category": "机器学习"
    },
    "lightgbm": {
        "url": "https://lightgbm.readthedocs.io/",
        "description": "高效的梯度提升决策树库",
        "category": "机器学习"
    },
    "catboost": {
        "url": "https://catboost.ai/",
        "description": "适合处理分类数据的梯度提升库",
        "category": "机器学习"
    },
    "matplotlib": {
        "url": "https://matplotlib.org/",
        "description": "数据可视化",
        "category": "数据处理与可视化"
    },
    "seaborn": {
        "url": "https://seaborn.pydata.org/",
        "description": "基于 Matplotlib 的数据可视化库",
        "category": "数据处理与可视化"
    },
    "plotly": {
        "url": "https://plotly.com/",
        "description": "交互式图表库",
        "category": "数据处理与可视化"
    },
    "scipy": {
        "url": "https://scipy.org/",
        "description": "科学计算库",
        "category": "数据处理与可视化"
    },
    "statsmodels": {
        "url": "https://www.statsmodels.org/",
        "description": "统计建模和计量经济学",
        "category": "数据处理与可视化"
    },
    "nltk": {
        "url": "https://www.nltk.org/",
        "description": "自然语言处理库",
        "category": "自然语言处理"
    },
    "spacy": {
        "url": "https://spacy.io/",
        "description": "高效的自然语言处理库",
        "category": "自然语言处理"
    },
    "transformers": {
        "url": "https://huggingface.co/transformers/",
        "description": "用于使用预训练的自然语言处理模型",
        "category": "自然语言处理"
    },
    "cv2": {
        "url": "https://opencv.org/",
        "description": "opencv-python 图像处理库",
        "category": "计算机视觉"
    },
    "PIL": {
        "url": "https://python-pillow.org/",
        "description": "Pillow 图像操作库",
        "category": "计算机视觉"
    },
    "gym": {
        "url": "https://www.gymlibrary.ml/",
        "description": "强化学习环境",
        "category": "强化学习"
    },
    "ray": {
        "url": "https://www.ray.io/",
        "description": "分布式计算，用于加速训练",
        "category": "分布式计算"
    },
    "joblib": {
        "url": "https://joblib.readthedocs.io/",
        "description": "并行计算与模型持久化",
        "category": "分布式计算"
    },
    "dask": {
        "url": "https://dask.org/",
        "description": "并行计算库，支持大规模数据处理",
        "category": "分布式计算"
    },
    "mlflow": {
        "url": "https://mlflow.org/",
        "description": "机器学习生命周期管理",
        "category": "机器学习管理"
    },
    "wandb": {
        "url": "https://wandb.ai/",
        "description": "实验跟踪与可视化",
        "category": "机器学习管理"
    },
    "hydra": {
        "url": "https://hydra.cc/",
        "description": "配置管理工具",
        "category": "机器学习管理"
    },
    "optuna": {
        "url": "https://optuna.org/",
        "description": "超参数优化库",
        "category": "机器学习管理"
    },
    "pycaret": {
        "url": "https://pycaret.org/",
        "description": "低代码机器学习库",
        "category": "机器学习管理"
    },
    "onnx": {
        "url": "https://onnx.ai/",
        "description": "用于深度学习模型的开放式神经网络交换格式",
        "category": "深度学习"
    },
    "albumentations": {
        "url": "https://albumentations.ai/",
        "description": "图像增强库",
        "category": "计算机视觉"
    },
    "tqdm": {
        "url": "https://tqdm.github.io/",
        "description": "进度条显示库",
        "category": "其他"
    },
    "librosa": {
        "url": "https://librosa.org/",
        "description": "用于音频和音乐信号分析的 Python 包，提供功能丰富的音频处理工具",
        "category": "音频处理"
    },
    "mir_eval": {
        "url": "https://craffel.github.io/mir_eval/",
        "description": "用于音乐信息检索 (MIR) 任务的评估库，支持一系列音乐分析基准的评估功能",
        "category": "音乐信息检索"
    }
}

def check_versions(libraries=ml_dl_libraries):
    """显示指定库的版本信息"""
    versions = {}
    for lib in libraries.keys():
        try:
            # module = importlib.import_module(lib)
            lib_version = version(lib)
            versions[lib] = lib_version
        except PackageNotFoundError:
            versions[lib] = 'Not installed'
        except ModuleNotFoundError:
            versions[lib] = 'Not installed'
    return versions

def check_all_versions(all_columns=False):
    """显示所有常用库的版本信息"""
    results = check_versions(ml_dl_libraries)

    console = Console()
    table = Table(show_header=True, header_style="bold magenta")
    table.add_column("Library", style="dim", width=16)
    table.add_column("Description", justify="left",width=30)
    if all_columns:
        table.add_column("Website", justify="left",width=30)
        table.add_column("Category", justify="left",width=20)
    table.add_column("Version", justify="right",width=20)
    for key, value in results.items():
        desc = ml_dl_libraries[key]["description"]
        site = ml_dl_libraries[key]["url"]
        category = ml_dl_libraries[key]["category"]
        if not value == 'Not installed':
            value = ':smiley: ' + '[red]' + value + '[/red]'
        if all_columns:
            table.add_row(key, desc, site,category, value)
        else:
            table.add_row(key, desc,  value)
    console.print(table)


def check_all_installed():
    """检查并显示所有已安装的 Python 库和版本信息"""
    console = Console()
    table = Table(show_header=True, header_style="bold magenta")
    table.add_column("Library", style="dim", width=30)
    table.add_column("Version", justify="right", width=15)

    # 获取所有已安装库的信息
    installed_packages = sorted((dist.metadata["Name"], dist.version) for dist in distributions())

    for name, version in installed_packages:
        table.add_row(name, version)

    console.print(table)


def get_latest_version(package_name):
    """查询 PyPI 获取指定包的最新版本"""
    url = f"https://pypi.org/pypi/{package_name}/json"
    try:
        response = requests.get(url, timeout=5)
        if response.status_code == 200:
            return response.json()["info"]["version"]
        else:
            return "Not found"
    except requests.RequestException:
        return "Error"


def check_all_installed_with_latest():
    """显示所有已安装库的当前版本和最新版本"""
    console = Console()
    table = Table(show_header=True, header_style="bold magenta")
    table.add_column("Library", style="dim", width=30)
    table.add_column("Installed Version", justify="right", width=15)
    table.add_column("Latest Version", justify="right", width=15)

    # 获取所有已安装库的名称和版本
    installed_packages = sorted((dist.metadata["Name"], dist.version) for dist in distributions())

    for name, installed_version in installed_packages:
        latest_version = get_latest_version(name)
        table.add_row(name, installed_version, latest_version)

    console.print(table)


# 调用函数显示所有已安装库的当前版本和最新版本
# check_all_installed_with_latest()
# 调用函数显示所有已安装库和版本
# check_all_installed()
# check_all_versions()



import matplotlib
import matplotlib.font_manager as fm
import matplotlib.pyplot as plt
import sys
import os
# Add the directory containing WebGetter.py to the Python path
sys.path.append(os.path.dirname(__file__))
from WebGetter import Wget

def is_chinese_font(font_path):
    """测试字体是否支持中文"""
    try:
        font = fm.FontProperties(fname=font_path)
        # 尝试渲染中文字符
        fig, ax = plt.subplots()
        ax.text(0.5, 0.5, '中文测试', fontproperties=font)
        plt.close(fig)
        return True
    except:
        return False

def fonts(show_demo = False):
    # 获取所有字体
    all_fonts = fm.findSystemFonts()
    chinese_fonts = []

    # 测试每个字体
    for font_path in all_fonts:
        try:
            if is_chinese_font(font_path):
                font = fm.FontProperties(fname=font_path)
                font_name = font.get_name()
                chinese_fonts.append({
                    'name': font_name,
                    'path': font_path
                })
        except:
            continue

    # 打印支持中文的字体
    print("\n支持中文的字体:")
    for font in chinese_fonts:
        print(f"字体名称: {font['name']}")
        print(f"字体路径: {font['path']}\n")

    # 测试显示
    if chinese_fonts and show_demo == True:
        plt.figure(figsize=(15, len(chinese_fonts)))
        for i, font in enumerate(chinese_fonts):
            plt.text(0.1, 1 - (i * 0.1),
                     f'这是{font["name"]}字体的中文显示测试',
                     fontproperties=fm.FontProperties(fname=font['path']),
                     fontsize=12)
        plt.axis('off')
        plt.show()

def simhei():
    # 创建下载器实例
    downloader = Wget(
        url='https://gitee.com/lincoln/fonts/raw/master/SimHei.ttf',
        output_dir='.',
        filename='SimHei.ttf',
    )
    downloader.download()

def matplotlib_font_init(show_demo = False):
    # 判断，同级文件夹下是否存在 SimHei.ttf 文件，如果没有，执行 simhei() 函数下载
    if not os.path.exists('./SimHei.ttf'):
        simhei()
    # 设置中文字体和样式
    matplotlib.font_manager.fontManager.addfont('./SimHei.ttf')
    matplotlib.rc('font', family='SimHei')
    # plt.rcParams['font.sans-serif'] = ['simhei']
    plt.rcParams['axes.unicode_minus'] = False

    # 测试显示
    if show_demo == True:
        plt.figure(figsize=(8, 6))
        plt.text(0.5, 0.5, '这是中文测试', fontsize=20)
        plt.show()


import sys
import os

def run_cmd(cmd):
    """
    Run command in the system.
    :param cmd: command to run
    :return: None
    """
    print('Running command:', cmd)
    p = os.popen(cmd)
    results = p.read()    #使用read()获取到一个字符串
    results_lst = results.split('\n')
    for line in results_lst:
        print(line)

def running_os():
    """
    Return the running environment.
    """
    env = sys.platform
    if env == 'win32':
        return 'Windows'
    elif env == 'linux':
        return 'Linux'
    elif env == 'darwin':
        return 'MacOS'
    else:
        return 'Unknown'

def running_python_version():
    """
    Return the running Python version.
    :return:
    """
    return sys.version

def running_python_path():
    """
    Return the running Python path.
    :return:
    """
    return sys.executable

def running_python_version_info():
    """
    Return the running Python version info.
    :return:
    """
    return sys.version_info

def running():
    """
    Return the running environment, Python version, Python path and Python version info.
    :return:
    """
    print("*"*40)
    print('Running environment:', running_os())
    print('Running Python version:', running_python_version())
    print('Running Python path:', running_python_path())
    print('Running Python version info:', running_python_version_info())
    print("*" * 40)
    return running_os(), running_python_version(), running_python_path(), running_python_version_info()

def install_ipykernel():
    """
    Install ipykernel.
    """
    # !pip install ipykernel
    run_cmd('pip install ipykernel')

def jupyter_kernel_list():
    """
    List the installed Jupyter kernels.
    """
    # !jupyter kernelspec list
    run_cmd('jupyter kernelspec list')

def install_kernel():
    """
    Install the kernel.
    """
    # !python -m ipykernel install --user --name=python3
    jupyter_kernel_list()
    install_ipykernel()
    running()
    python_path = running_python_path()
    print("Please input the new kernel name and display name.")
    kernel_name = input("Please input the kernel name:")
    kernel_display_name = input("Please input the kernel display name:")
    run_cmd(python_path + ' -m ipykernel install --user --name=' + kernel_name + ' --display-name=' + kernel_display_name)

def no_warning():
    import warnings
    warnings.filterwarnings('ignore')



def is_Chinese(ch):
    if '\u4e00' <= ch <= '\u9fff':
        return True
    return False


def algin(title_key, max_english):
    chinese_count = 0
    english_count = 0
    for j in str(title_key):
        if is_Chinese(j):
            chinese_count = chinese_count + 1
        else:
            english_count = english_count + 1

    temp = max_english - english_count
    while temp > 0:
        title_key = title_key + ' '
        temp = temp - 1
    title_key = title_key.ljust(7, chr(12288))
    # print(title_key + '-')
    return title_key


if __name__ == '__main__':
    algin("a一二三", 3)
    algin("aa一二三", 3)
    algin("aaa一二三", 3)
    algin("a一二三aa", 3)


import pandas as pd
import numpy as np
import seaborn as sns
from scipy.stats import skew, kurtosis, zscore
import missingno as msno
import os
import matplotlib.pyplot as plt
from matplotlib import rcParams

# 设置字体为 SimHei
rcParams['font.sans-serif'] = ['SimHei']  # 设置默认字体
rcParams['axes.unicode_minus'] = False   # 解决负号显示问题

def analyze_data(file_path):
    # 获取文件名和扩展名
    file_name = os.path.splitext(os.path.basename(file_path))[0]
    file_extension = os.path.splitext(file_path)[1].lower()

    # 判断文件格式并选择读取方法
    if file_extension == '.csv':
        df = pd.read_csv(file_path)
    elif file_extension in ['.xls', '.xlsx']:
        df = pd.read_excel(file_path)
    elif file_extension == '.json':
        df = pd.read_json(file_path)
    else:
        raise ValueError("不支持的文件格式。请使用 CSV、Excel 或 JSON 格式的文件。")

    # 设置输出文件名称
    output_text_file = f"{file_name}_analysis_output.txt"

    # 开始分析并输出结果
    with open(output_text_file, "w") as f:
        def write_output(text):
            # 在文件和控制台同时输出
            print(text)
            f.write(text + "\n")

        # 1. 数据基本信息
        write_output("=== DataFrame 基本信息 ===")
        df_info = df.info(buf=f)

        # 缺失值数量和比例
        write_output("\n=== DataFrame 缺失值统计 ===")
        missing_values = df.isnull().sum()
        missing_percentage = df.isnull().mean() * 100
        missing_df = pd.DataFrame({'缺失值数量': missing_values, '缺失值比例': missing_percentage})
        write_output(str(missing_df))

        # 唯一值数量
        write_output("\n=== DataFrame 唯一值数量 ===")
        unique_values = df.nunique()
        write_output(str(unique_values))

        # 2. 数值型特征的描述性统计分析
        write_output("\n=== 数值型特征的描述性统计 ===")
        desc_stats = df.describe()
        write_output(str(desc_stats))

        # 偏度和峰度分析
        write_output("\n=== 偏度和峰度分析 ===")
        for column in df.select_dtypes(include=['int64', 'float64']).columns:
            skewness = skew(df[column].dropna())
            kurtosis_val = kurtosis(df[column].dropna())

            # 偏度
            write_output(f"\n{column} 的偏度 (Skewness): {skewness}")
            if skewness > 0:
                write_output("解释: 偏度为正值，表示数据右偏，数据分布的右尾较长。")
            elif skewness < 0:
                write_output("解释: 偏度为负值，表示数据左偏，数据分布的左尾较长。")
            else:
                write_output("解释: 偏度接近零，数据呈对称分布。")

            # 峰度
            write_output(f"{column} 的峰度 (Kurtosis): {kurtosis_val}")
            if kurtosis_val > 0:
                write_output("解释: 峰度为正值，表示数据分布尖峰较高，尾部更厚（可能存在更多极端值）。")
            elif kurtosis_val < 0:
                write_output("解释: 峰度为负值，表示数据分布较平坦，尾部较薄。")
            else:
                write_output("解释: 峰度接近零，数据呈正态分布形态。")

        # 3. 分类特征的频率分布
        for column in df.select_dtypes(include=['object', 'category']).columns:
            write_output(f"\n=== {column} 的频率分布 ===")
            write_output(str(df[column].value_counts()))
            write_output(f"解释: {column} 各类别的出现次数，便于了解该列的类别是否均衡。")

        # 6. 异常值检测
        write_output("=== 异常值检测 ===")
        for column in df.select_dtypes(include=['int64', 'float64']).columns:
            df['zscore_' + column] = zscore(df[column].dropna())
            outliers_zscore = df[np.abs(df['zscore_' + column]) > 3]
            write_output(f"\n{column} 的 Z-score 异常值数量：{len(outliers_zscore)}")
            if len(outliers_zscore) > 0:
                write_output("解释: 使用 Z-score 检测到异常值，通常 Z-score > 3 的值可能是异常值。")

            Q1 = df[column].quantile(0.25)
            Q3 = df[column].quantile(0.75)
            IQR = Q3 - Q1
            outliers_iqr = df[(df[column] < (Q1 - 1.5 * IQR)) | (df[column] > (Q3 + 1.5 * IQR))]
            write_output(f"{column} 的 IQR 异常值数量：{len(outliers_iqr)}")
            if len(outliers_iqr) > 0:
                write_output("解释: 使用 IQR 检测到异常值，低于 Q1 - 1.5 * IQR 或高于 Q3 + 1.5 * IQR 的值可能是异常值。")

        # 7. 数值特征的相关性分析
        write_output("\n=== 数值特征的相关系数矩阵 ===")
        correlation_matrix = df.corr()
        write_output(str(correlation_matrix))
        write_output(
            "解释: 相关性矩阵用于显示数值特征之间的相关程度。高相关性可能表明特征间的共线性，可在模型训练中考虑消除冗余特征。")

    # 生成图表并直接显示
    for column in df.select_dtypes(include=['int64', 'float64']).columns:
        plt.figure(figsize=(10, 4))
        sns.histplot(df[column], kde=True)
        plt.title(f"{column}的分布图 - 直方图和密度图")
        plt.xlabel(column)
        plt.ylabel("频率")
        plt.show()  # 显示直方图

        sns.boxplot(x=df[column])
        plt.title(f"{column}的箱线图 - 显示中位数、四分位数和异常值")
        plt.xlabel(column)
        plt.show()  # 显示箱线图

    plt.figure(figsize=(12, 8))
    sns.heatmap(correlation_matrix, annot=True, cmap='coolwarm', fmt=".2f")
    plt.title("数值特征的相关性矩阵")
    plt.show()  # 显示相关性热图

    msno.heatmap(df)
    plt.title("缺失值热图")
    plt.show()  # 显示缺失值热图

    write_output("分析已完成，结果已显示在控制台。")

# 使用示例
# analyze_data("your_data_file.csv")


import requests
import os
import sys
from urllib.parse import urlparse
from tqdm import tqdm


class Wget:
    def __init__(self, url, output_dir='.', filename=None):
        self.url = url
        self.output_dir = output_dir
        self.filename = filename or self._get_filename_from_url()

    def _get_filename_from_url(self):
        """从 URL 中获取文件名"""
        parsed = urlparse(self.url)
        return os.path.basename(parsed.path) or 'downloaded_file'

    def _create_output_dir(self):
        """创建输出目录"""
        if not os.path.exists(self.output_dir):
            os.makedirs(self.output_dir)

    def download(self):
        """下载文件"""
        try:
            # 发送 HEAD 请求获取文件大小
            response = requests.head(self.url)
            total_size = int(response.headers.get('content-length', 0))

            # 创建输出目录
            self._create_output_dir()
            output_path = os.path.join(self.output_dir, self.filename)

            # 发送 GET 请求下载文件
            response = requests.get(self.url, stream=True)
            response.raise_for_status()  # 检查响应状态

            # 使用 tqdm 显示进度条
            with open(output_path, 'wb') as f, tqdm(
                    desc=self.filename,
                    total=total_size,
                    unit='iB',
                    unit_scale=True,
                    unit_divisor=1024,
            ) as pbar:
                for data in response.iter_content(chunk_size=1024):
                    size = f.write(data)
                    pbar.update(size)

            print(f"\n下载完成: {output_path}")
            return True

        except requests.exceptions.RequestException as e:
            print(f"下载错误: {str(e)}")
            return False
        except Exception as e:
            print(f"发生错误: {str(e)}")
            return False


# def main():
#     """主函数"""
#     if len(sys.argv) < 2:
#         print("使用方法: python wget.py <URL> [输出目录] [文件名]")
#         sys.exit(1)
#
#     url = sys.argv[1]
#     output_dir = sys.argv[2] if len(sys.argv) > 2 else '.'
#     filename = sys.argv[3] if len(sys.argv) > 3 else None
#
#     downloader = Wget(url, output_dir, filename)
#     downloader.download()
#
#
# if __name__ == "__main__":
#     main()


import os  # 确保os模块在文件开头导入
import base64
import json
import logging
from datetime import datetime
from io import BytesIO
from typing import Dict, List, Any

import matplotlib.pyplot as plt
import matplotlib.font_manager as fm
from matplotlib import rcParams
import numpy as np
import pandas as pd
import seaborn as sns
from jinja2 import Environment, FileSystemLoader
from scipy.stats import skew, kurtosis, zscore, normaltest, shapiro
from sklearn.preprocessing import StandardScaler
from sklearn.decomposition import PCA
from statsmodels.stats.outliers_influence import variance_inflation_factor

# 配置日志
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
)
logger = logging.getLogger(__name__)

# 获取当前文件所在目录
current_dir = os.path.dirname(os.path.abspath(__file__))
font_path = os.path.join(current_dir, 'SimHei.ttf')

# 添加字体文件到 matplotlib 的字体管理器
if os.path.exists(font_path):
    font_prop = fm.FontProperties(fname=font_path)
    rcParams['font.family'] = 'sans-serif'
    rcParams['font.sans-serif'] = ['SimHei'] + rcParams['font.sans-serif']
    rcParams['axes.unicode_minus'] = False
    logger.info(f"已加载字体文件: {font_path}")
else:
    logger.warning(f"警告: 未找到字体文件: {font_path}")
    # 使用系统默认字体
    rcParams['font.sans-serif'] = ['SimHei', 'Microsoft YaHei', 'WenQuanYi Micro Hei', 'sans-serif']
    rcParams['axes.unicode_minus'] = False

class DataProcessor:
    """数据处理工具类"""
    
    @staticmethod
    def validate_dataframe(df: pd.DataFrame) -> None:
        """验证DataFrame的有效性"""
        if not isinstance(df, pd.DataFrame):
            raise TypeError("输入必须是pandas DataFrame")
        if df.empty:
            raise ValueError("DataFrame不能为空")
    
    @staticmethod
    def get_column_types(df: pd.DataFrame) -> Dict[str, List[str]]:
        """获取列类型信息"""
        return {
            'numeric_cols': df.select_dtypes(include=['number']).columns.tolist(),
            'categorical_cols': df.select_dtypes(include=['object', 'category']).columns.tolist(),
            'datetime_cols': df.select_dtypes(include=['datetime']).columns.tolist()
        }
    
    @staticmethod
    def process_value(v: Any) -> Any:
        """处理单个值，转换为可序列化格式"""
        if isinstance(v, np.ndarray):
            return v.tolist()
        if isinstance(v, (np.int64, np.int32)):
            return int(v)
        if isinstance(v, (np.float64, np.float32)):
            if np.isnan(v) or np.isinf(v):
                return None
            return float(v)
        if isinstance(v, datetime):
            return v.isoformat()
        if isinstance(v, pd.Series):
            return v.to_dict()
        if isinstance(v, pd.DataFrame):
            return v.to_dict('records')
        try:
            if pd.isna(v):
                return None
        except (TypeError, ValueError):
            pass
        return v
    
    @staticmethod
    def process_data(obj: Any) -> Any:
        """递归处理数据，转换为可序列化格式"""
        if isinstance(obj, dict):
            return {k: DataProcessor.process_data(v) for k, v in obj.items()}
        elif isinstance(obj, (list, np.ndarray)):
            return [DataProcessor.process_data(item) for item in obj]
        else:
            return DataProcessor.process_value(obj)
    
    @staticmethod
    def chunk_dataframe(df: pd.DataFrame, chunk_size: int = 1000) -> List[pd.DataFrame]:
        """将DataFrame分块处理"""
        return [df[i:i + chunk_size] for i in range(0, len(df), chunk_size)]

class DataAnalyzer:
    """数据分析器主类"""
    
    def __init__(self, df: pd.DataFrame):
        # 验证输入数据
        DataProcessor.validate_dataframe(df)
        self.df = df.copy()
        
        # 获取列类型信息
        column_types = DataProcessor.get_column_types(self.df)
        self.numeric_cols = column_types['numeric_cols']
        self.categorical_cols = column_types['categorical_cols']
        self.datetime_cols = column_types['datetime_cols']
        
        # 初始化可视化工具
        self.visualizer = Visualizer(font_prop)
        
        # 初始化缓存
        self._correlation_cache = None
        self._multicollinearity_cache = None
    
    def __del__(self):
        """清理资源"""
        if hasattr(self, 'visualizer'):
            self.visualizer.clear_cache()
    
    def correlation_analysis(self) -> pd.DataFrame:
        """相关性分析"""
        if self._correlation_cache is None:
            self._correlation_cache = self.df[self.numeric_cols].corr()
        return self._correlation_cache
    
    def multicollinearity_analysis(self) -> pd.DataFrame:
        """多重共线性分析"""
        if self._multicollinearity_cache is None:
            vif_data = pd.DataFrame()
            vif_data["列名"] = self.numeric_cols
            vif_data["VIF"] = [variance_inflation_factor(self.df[self.numeric_cols].values, i) 
                              for i in range(len(self.numeric_cols))]
            self._multicollinearity_cache = vif_data
        return self._multicollinearity_cache
    
    def plot_distribution(self, column: str) -> str:
        """绘制分布图"""
        if column not in self.df.columns:
            raise ValueError(f"列名不存在: {column}")
        return self.visualizer.plot_distribution(self.df[column], column)
    
    def plot_correlation_heatmap(self) -> str:
        """绘制相关性热图"""
        return self.visualizer.plot_correlation_heatmap(self.correlation_analysis())
    
    def generate_report(self, output_html: str = "analysis_report.html") -> None:
        """生成分析报告"""
        try:
            # 获取数据集名称
            try:
                base_name = os.path.basename(output_html)
                name_without_ext = os.path.splitext(base_name)[0]
                suffixes_to_remove = ['_analysis', '_report', '_data']
                dataset_name = name_without_ext
                for suffix in suffixes_to_remove:
                    if dataset_name.endswith(suffix):
                        dataset_name = dataset_name[:-len(suffix)]
                dataset_name = dataset_name.replace('_', ' ').title()
            except Exception as e:
                logger.error(f"处理数据集名称时出错: {str(e)}")
                dataset_name = "未命名数据集"
            
            # 收集所有分析结果
            logger.info("收集分析结果...")
            analysis_results = {
                "dataset_name": dataset_name,
                "basic_stats": self.basic_statistics().to_dict('records'),
                "normality_test": self.normality_test().to_dict('records'),
                "missing_analysis": self.missing_value_analysis().to_dict('records'),
                "outlier_analysis": self.outlier_analysis().to_dict('records'),
                "duplicate_analysis": self.duplicate_analysis(),
                "correlation_matrix": self.correlation_analysis().to_dict(),
                "multicollinearity": self.multicollinearity_analysis().to_dict('records'),
                "pca_analysis": self.pca_analysis(),
                "plots": {
                    "distribution": {col: self.plot_distribution(col) for col in self.df.columns},
                    "correlation": self.plot_correlation_heatmap()
                },
                "interpretations": self.interpret_results()
            }
            
            # 获取包内模板目录的绝对路径
            template_dir = os.path.join(os.path.dirname(__file__), 'templates')
            static_dir = os.path.join(os.path.dirname(__file__), 'static')
            
            # 创建报告生成器并生成报告
            report_generator = ReportGenerator(template_dir, static_dir)
            report_generator.generate_report(analysis_results, output_html)
            
        except Exception as e:
            logger.error(f"生成报告时出错: {str(e)}")
            raise

    # ==================== 基础统计分析 ====================
    def basic_statistics(self) -> pd.DataFrame:
        """计算基本统计量"""
        # 获取基本统计信息
        stats_df = self.df.describe(include='all').transpose()
        
        # 添加缺失值相关信息
        stats_df['缺失值数量'] = self.df.isnull().sum()
        stats_df['缺失率 (%)'] = (self.df.isnull().mean() * 100).round(2)
        stats_df['唯一值数量'] = self.df.nunique()
        
        # 处理空值，将其转换为更易读的格式
        stats_df = stats_df.apply(lambda x: x.apply(lambda v: '空值' if pd.isna(v) else v))
        
        # 对数值列的统计量进行格式化
        numeric_cols = self.df.select_dtypes(include=['number']).columns
        for col in stats_df.index:
            if col in numeric_cols:
                for stat in ['mean', 'std', '25%', '50%', '75%', 'min', 'max']:
                    if stat in stats_df.columns and not isinstance(stats_df.at[col, stat], str):
                        stats_df.at[col, stat] = f"{stats_df.at[col, stat]:.2f}"
        
        return stats_df.reset_index().rename(columns={'index': '列名'})

    def normality_test(self) -> pd.DataFrame:
        """正态性检验"""
        results = []
        for col in self.numeric_cols:
            data = self.df[col].dropna()
            if len(data) > 3:  # 至少需要3个样本才能进行正态性检验
                # Shapiro-Wilk检验
                shapiro_stat, shapiro_p = shapiro(data)
                # D'Agostino-Pearson检验
                norm_stat, norm_p = normaltest(data)
                
                results.append({
                    '列名': col,
                    'Shapiro-Wilk统计量': shapiro_stat,
                    'Shapiro-Wilk p值': shapiro_p,
                    'D-Agostino-Pearson统计量': norm_stat,
                    'D-Agostino-Pearson p值': norm_p,
                    '是否正态分布': '是' if shapiro_p > 0.05 and norm_p > 0.05 else '否'
                })
        return pd.DataFrame(results)

    # ==================== 数据质量分析 ====================
    def missing_value_analysis(self) -> pd.DataFrame:
        """缺失值分析"""
        missing_counts = self.df.isnull().sum()
        missing_ratios = self.df.isnull().mean() * 100
        suggestions = []
        
        for col, ratio in missing_ratios.items():
            if ratio == 0:
                suggestion = "无缺失，无需处理"
            elif ratio < 5:
                suggestion = "填充缺失值，例如均值/众数"
            elif ratio < 50:
                suggestion = "视情况填充或丢弃列"
            else:
                suggestion = "考虑丢弃列"
            suggestions.append(suggestion)
        
        return pd.DataFrame({
            "列名": self.df.columns,
            "缺失值数量": missing_counts,
            "缺失率 (%)": missing_ratios,
            "建议处理方案": suggestions
        })

    def outlier_analysis(self) -> pd.DataFrame:
        """异常值分析"""
        results = []
        for col in self.numeric_cols:
            data = self.df[col].dropna()
            
            # Z-score方法
            z_scores = zscore(data)
            outliers_zscore = np.sum(np.abs(z_scores) > 3)
            zscore_explanation = self._get_zscore_interpretation(z_scores)
            
            # IQR方法
            Q1 = data.quantile(0.25)
            Q3 = data.quantile(0.75)
            IQR = Q3 - Q1
            lower_bound = Q1 - 1.5 * IQR
            upper_bound = Q3 + 1.5 * IQR
            outliers_iqr = np.sum((data < lower_bound) | (data > upper_bound))
            iqr_explanation = self._get_iqr_interpretation(data, Q1, Q3, IQR)
            
            # 箱线图方法
            outliers_box = np.sum((data < Q1 - 3 * IQR) | (data > Q3 + 3 * IQR))
            
            results.append({
                "列名": col,
                "Z-score异常值数量": outliers_zscore,
                "Z-score解释": zscore_explanation,
                "IQR异常值数量": outliers_iqr,
                "IQR解释": iqr_explanation,
                "箱线图异常值数量": outliers_box,
                "异常值比例 (%)": (max(outliers_zscore, outliers_iqr, outliers_box) / len(data) * 100).round(2)
            })
        
        return pd.DataFrame(results)

    def _get_zscore_interpretation(self, z_scores: np.ndarray) -> str:
        """生成Z-score方法的解释"""
        total_points = len(z_scores)
        extreme_outliers = np.sum(np.abs(z_scores) > 3)  # 极端异常值
        moderate_outliers = np.sum((np.abs(z_scores) > 2) & (np.abs(z_scores) <= 3))  # 中度异常值
        
        interpretation = []
        
        # 总体情况
        if extreme_outliers == 0 and moderate_outliers == 0:
            interpretation.append("数据分布较为集中，未检测到异常值。")
        else:
            interpretation.append(f"共检测到 {extreme_outliers + moderate_outliers} 个异常值，占总数据的 {((extreme_outliers + moderate_outliers) / total_points * 100):.2f}%。")
        
        # 详细解释
        if extreme_outliers > 0:
            interpretation.append(f"其中 {extreme_outliers} 个为极端异常值（|Z-score| > 3），占总数据的 {extreme_outliers / total_points * 100:.2f}%。")
        if moderate_outliers > 0:
            interpretation.append(f"另有 {moderate_outliers} 个为中度异常值（2 < |Z-score| ≤ 3），占总数据的 {moderate_outliers / total_points * 100:.2f}%。")
        
        # 建议
        if extreme_outliers > 0:
            interpretation.append("建议检查这些极端异常值的合理性，必要时进行处理。")
        elif moderate_outliers > 0:
            interpretation.append("建议关注这些中度异常值，确认其是否合理。")
        
        return " ".join(interpretation)

    def _get_iqr_interpretation(self, data: pd.Series, Q1: float, Q3: float, IQR: float) -> str:
        """生成IQR方法的解释"""
        lower_bound = Q1 - 1.5 * IQR
        upper_bound = Q3 + 1.5 * IQR
        mild_outliers = np.sum((data < lower_bound) | (data > upper_bound))
        
        # 计算四分位距的分布
        q1_percentile = np.percentile(data, 25)
        q3_percentile = np.percentile(data, 75)
        median = np.median(data)
        
        interpretation = []
        
        # 总体情况
        if mild_outliers == 0:
            interpretation.append("数据分布较为均匀，未检测到异常值。")
        else:
            interpretation.append(f"检测到 {mild_outliers} 个异常值，占总数据的 {mild_outliers / len(data) * 100:.2f}%。")
        
        # 数据分布特征
        interpretation.append(f"数据的中位数为 {median:.2f}，")
        if Q3 - Q1 < np.std(data):
            interpretation.append("四分位距较小，说明数据较为集中；")
        else:
            interpretation.append("四分位距较大，说明数据较为分散；")
        
        # 异常值范围
        interpretation.append(f"异常值范围为：小于 {lower_bound:.2f} 或大于 {upper_bound:.2f}。")
        
        # 建议
        if mild_outliers > 0:
            if mild_outliers / len(data) > 0.1:  # 异常值比例超过10%
                interpretation.append("异常值比例较高，建议进行详细检查并考虑是否需要处理。")
            else:
                interpretation.append("异常值比例较低，建议检查这些值的合理性。")
        
        return " ".join(interpretation)

    def duplicate_analysis(self) -> Dict[str, Any]:
        """重复值分析"""
        duplicate_rows = self.df.duplicated().sum()
        duplicate_ratio = (duplicate_rows / len(self.df) * 100).round(2)
        
        return {
            "重复行数量": duplicate_rows,
            "重复率 (%)": duplicate_ratio,
            "建议": "建议删除重复行" if duplicate_ratio > 5 else "重复率较低，可保留"
        }

    # ==================== 高级统计分析 ====================
    def pca_analysis(self) -> Dict[str, Any]:
        """主成分分析"""
        if len(self.numeric_cols) < 2:
            return {"error": "需要至少两个数值型变量进行PCA分析"}
            
        # 只对数值型变量进行PCA
        scaler = StandardScaler()
        scaled_data = scaler.fit_transform(self.df[self.numeric_cols])
        
        # 执行PCA
        pca = PCA()
        pca.fit(scaled_data)
        
        # 计算累计方差贡献率
        cumulative_variance = np.cumsum(pca.explained_variance_ratio_)
        
        return {
            "主成分数量": len(self.numeric_cols),
            "各主成分方差贡献率": pca.explained_variance_ratio_,
            "累计方差贡献率": cumulative_variance,
            "建议保留主成分数量": np.argmax(cumulative_variance >= 0.95) + 1
        }

    # ==================== 结果解读 ====================
    def interpret_results(self) -> Dict[str, Any]:
        """解读所有分析结果"""
        interpretations = {
            "basic_info": self._interpret_basic_info(),
            "data_quality": self._interpret_data_quality(),
            "statistical_analysis": self._interpret_statistical_analysis(),
            "recommendations": self._generate_recommendations()
        }
        return interpretations

    def _interpret_basic_info(self) -> Dict[str, Any]:
        """解读基本信息"""
        total_rows, total_cols = self.df.shape
        numeric_count = len(self.numeric_cols)
        categorical_count = len(self.categorical_cols)
        datetime_count = len(self.datetime_cols)

        # 获取各类型变量的详细信息
        numeric_vars = []
        for col in self.numeric_cols:
            sample = self.df[col].iloc[0]
            if pd.notna(sample):  # 确保样本值不是NaN
                numeric_vars.append({
                    "name": col,
                    "sample": f"{sample:.4f}" if isinstance(sample, (int, float)) else str(sample)
                })

        categorical_vars = []
        for col in self.categorical_cols:
            sample = self.df[col].iloc[0]
            if pd.notna(sample):  # 确保样本值不是NaN
                categorical_vars.append({
                    "name": col,
                    "sample": str(sample)
                })

        datetime_vars = []
        for col in self.datetime_cols:
            sample = self.df[col].iloc[0]
            if pd.notna(sample):  # 确保样本值不是NaN
                datetime_vars.append({
                    "name": col,
                    "sample": str(sample)
                })

        print("变量详情：")
        print(f"数值型变量: {numeric_vars}")
        print(f"分类型变量: {categorical_vars}")
        print(f"时间型变量: {datetime_vars}")

        return {
            "数据集规模": f"数据集包含 {total_rows} 行和 {total_cols} 列。",
            "变量类型": f"其中包含 {numeric_count} 个数值型变量、{categorical_count} 个分类型变量和 {datetime_count} 个时间型变量。",
            "说明": "这个规模的数据集适合进行统计分析，但需要注意数据质量和变量之间的关系。",
            "变量详情": {
                "数值型变量": numeric_vars,
                "分类型变量": categorical_vars,
                "时间型变量": datetime_vars
            }
        }

    def _interpret_data_quality(self) -> Dict[str, Any]:
        """解读数据质量分析结果"""
        # 缺失值分析
        missing_stats = self.missing_value_analysis()
        missing_interpretation = {
            "总体情况": f"数据集中共有 {len(missing_stats)} 个变量，其中 {sum(missing_stats['缺失值数量'] > 0)} 个变量存在缺失值。",
            "缺失值分布": "缺失值分布情况如下：",
            "details": []
        }

        for _, row in missing_stats.iterrows():
            if row['缺失值数量'] > 0:
                missing_interpretation["details"].append({
                    "变量": row['列名'],
                    "缺失情况": f"缺失 {row['缺失值数量']} 个值，缺失率为 {row['缺失率 (%)']:.2f}%",
                    "建议": row['建议处理方案']
                })

        # 异常值分析
        outlier_stats = self.outlier_analysis()
        outlier_interpretation = {
            "总体情况": f"数据集中共有 {len(outlier_stats)} 个数值型变量，其中 {sum(outlier_stats['异常值比例 (%)'] > 0)} 个变量存在异常值。",
            "异常值分布": "异常值分布情况如下：",
            "details": []
        }

        for _, row in outlier_stats.iterrows():
            if row['异常值比例 (%)'] > 0:
                outlier_interpretation["details"].append({
                    "变量": row['列名'],
                    "Z-score分析": row['Z-score解释'],
                    "IQR分析": row['IQR解释'],
                    "异常值比例": f"{row['异常值比例 (%)']:.2f}%",
                    "建议": "建议根据Z-score和IQR的分析结果，综合考虑是否需要处理异常值。"
                })

        # 重复值分析
        duplicate_stats = self.duplicate_analysis()
        duplicate_interpretation = {
            "总体情况": f"数据集中存在 {duplicate_stats['重复行数量']} 行重复数据，重复率为 {duplicate_stats['重复率 (%)']:.2f}%。",
            "建议": duplicate_stats['建议']
        }

        return {
            "缺失值分析": missing_interpretation,
            "异常值分析": outlier_interpretation,
            "重复值分析": duplicate_interpretation
        }

    def _interpret_statistical_analysis(self) -> Dict[str, Any]:
        """解读统计分析结果"""
        # 正态性检验解读
        normality_stats = self.normality_test()
        normality_interpretation = {
            "总体情况": f"对 {len(normality_stats)} 个数值型变量进行了正态性检验。",
            "检验结果": "检验结果如下：",
            "details": []
        }

        for _, row in normality_stats.iterrows():
            normality_interpretation["details"].append({
                "变量": row['列名'],
                "是否正态分布": row['是否正态分布'],
                "解释": "该变量服从正态分布，可以使用参数检验方法。" if row['是否正态分布'] == '是' else "该变量不服从正态分布，建议使用非参数检验方法。"
            })

        # 峰度和偏度解读
        skew_kurt_stats = self._analyze_skew_kurtosis()
        skew_kurt_interpretation = {
            "总体情况": "对数值型变量进行了峰度和偏度分析，结果如下：",
            "details": []
        }

        for _, row in skew_kurt_stats.iterrows():
            # 偏度解释
            skewness = row['偏度']
            if abs(skewness) < 0.5:
                skew_explanation = "数据分布接近对称"
            elif skewness > 0:
                skew_explanation = "数据右偏，右尾较长，可能存在较大的异常值"
            else:
                skew_explanation = "数据左偏，左尾较长，可能存在较小的异常值"

            # 峰度解释
            kurtosis_val = row['峰度']
            if abs(kurtosis_val) < 0.5:
                kurtosis_explanation = "数据分布接近正态分布"
            elif kurtosis_val > 0:
                kurtosis_explanation = "数据分布尖峰，尾部较厚，存在较多极端值"
            else:
                kurtosis_explanation = "数据分布平坦，尾部较薄，极端值较少"

            skew_kurt_interpretation["details"].append({
                "变量": row['列名'],
                "偏度": f"{skewness:.3f}",
                "偏度解释": skew_explanation,
                "峰度": f"{kurtosis_val:.3f}",
                "峰度解释": kurtosis_explanation,
                "建议": self._get_skew_kurtosis_recommendation(skewness, kurtosis_val)
            })

        # 相关性分析解读
        corr_matrix = self.correlation_analysis()
        corr_interpretation = {
            "总体情况": "变量间的相关性分析结果如下：",
            "强相关变量": [],
            "中等相关变量": [],
            "弱相关变量": []
        }

        for i in range(len(corr_matrix.columns)):
            for j in range(i+1, len(corr_matrix.columns)):
                corr = corr_matrix.iloc[i, j]
                if abs(corr) >= 0.7:
                    corr_interpretation["强相关变量"].append({
                        "变量对": f"{corr_matrix.columns[i]} - {corr_matrix.columns[j]}",
                        "相关系数": f"{corr:.2f}",
                        "解释": "这两个变量存在强相关性，可能存在多重共线性问题。"
                    })
                elif abs(corr) >= 0.3:
                    corr_interpretation["中等相关变量"].append({
                        "变量对": f"{corr_matrix.columns[i]} - {corr_matrix.columns[j]}",
                        "相关系数": f"{corr:.2f}",
                        "解释": "这两个变量存在中等程度的相关性。"
                    })
                elif abs(corr) >= 0.1:
                    corr_interpretation["弱相关变量"].append({
                        "变量对": f"{corr_matrix.columns[i]} - {corr_matrix.columns[j]}",
                        "相关系数": f"{corr:.2f}",
                        "解释": "这两个变量存在弱相关性。"
                    })

        # PCA分析解读
        pca_stats = self.pca_analysis()
        pca_interpretation = {
            "总体情况": f"主成分分析结果显示，数据集中有 {pca_stats['主成分数量']} 个主成分。",
            "建议保留主成分数": f"建议保留 {pca_stats['建议保留主成分数量']} 个主成分，可以解释 {pca_stats['累计方差贡献率'][pca_stats['建议保留主成分数量']-1]*100:.2f}% 的方差。",
            "解释": "主成分分析可以帮助降维，减少变量间的相关性。"
        }

        return {
            "正态性检验": normality_interpretation,
            "峰度偏度分析": skew_kurt_interpretation,
            "相关性分析": corr_interpretation,
            "主成分分析": pca_interpretation
        }

    def _analyze_skew_kurtosis(self) -> pd.DataFrame:
        """分析数值型变量的峰度和偏度"""
        results = []
        for col in self.numeric_cols:
            data = self.df[col].dropna()
            if len(data) > 0:
                skewness = skew(data)
                kurtosis_val = kurtosis(data)
                results.append({
                    "列名": col,
                    "偏度": skewness,
                    "峰度": kurtosis_val
                })
        return pd.DataFrame(results)

    def _get_skew_kurtosis_recommendation(self, skewness: float, kurtosis: float) -> str:
        """根据峰度和偏度生成建议"""
        recommendations = []
        
        # 偏度建议
        if abs(skewness) >= 1:
            recommendations.append("数据严重偏斜，建议进行数据转换（如对数转换）")
        elif abs(skewness) >= 0.5:
            recommendations.append("数据存在一定偏斜，可以考虑进行数据转换")
        
        # 峰度建议
        if abs(kurtosis) >= 2:
            recommendations.append("数据分布与正态分布差异较大，建议使用稳健统计方法")
        elif abs(kurtosis) >= 1:
            recommendations.append("数据分布与正态分布有一定差异，建议检查异常值")
        
        if not recommendations:
            recommendations.append("数据分布接近正态，可以使用常规统计方法")
        
        return "；".join(recommendations)

    def _generate_recommendations(self) -> List[Dict[str, str]]:
        """生成数据分析和处理建议"""
        recommendations = []

        # 数据质量建议
        missing_stats = self.missing_value_analysis()
        for _, row in missing_stats.iterrows():
            if row['缺失率 (%)'] > 0:
                recommendations.append({
                    "类型": "数据质量",
                    "问题": f"变量 {row['列名']} 存在缺失值",
                    "建议": row['建议处理方案']
                })

        outlier_stats = self.outlier_analysis()
        for _, row in outlier_stats.iterrows():
            if row['异常值比例 (%)'] > 5:
                recommendations.append({
                    "类型": "数据质量",
                    "问题": f"变量 {row['列名']} 存在较多异常值",
                    "建议": "建议检查异常值的合理性，必要时进行处理。"
                })

        # 统计分析建议
        normality_stats = self.normality_test()
        for _, row in normality_stats.iterrows():
            if row['是否正态分布'] == '否':
                recommendations.append({
                    "类型": "统计分析",
                    "问题": f"变量 {row['列名']} 不服从正态分布",
                    "建议": "建议使用非参数检验方法进行分析。"
                })

        # 峰度偏度建议
        skew_kurt_stats = self._analyze_skew_kurtosis()
        for _, row in skew_kurt_stats.iterrows():
            if abs(row['偏度']) >= 1 or abs(row['峰度']) >= 2:
                recommendations.append({
                    "类型": "统计分析",
                    "问题": f"变量 {row['列名']} 的分布严重偏离正态分布",
                    "建议": self._get_skew_kurtosis_recommendation(row['偏度'], row['峰度'])
                })

        corr_matrix = self.correlation_analysis()
        for i in range(len(corr_matrix.columns)):
            for j in range(i+1, len(corr_matrix.columns)):
                corr = corr_matrix.iloc[i, j]
                if abs(corr) >= 0.7:
                    recommendations.append({
                        "类型": "统计分析",
                        "问题": f"变量 {corr_matrix.columns[i]} 和 {corr_matrix.columns[j]} 存在强相关性",
                        "建议": "建议考虑删除其中一个变量或使用主成分分析降维。"
                    })

        return recommendations

class Visualizer:
    """数据可视化工具类"""
    
    def __init__(self, font_prop: fm.FontProperties):
        self.font_prop = font_prop
        self._figure_cache = {}
    
    def _create_figure(self, figsize: tuple = (10, 6)) -> plt.Figure:
        """创建新的图表"""
        return plt.figure(figsize=figsize)
    
    def _close_figure(self, fig: plt.Figure) -> None:
        """关闭图表"""
        plt.close(fig)
    
    def plot_distribution(self, data: pd.Series, column: str) -> str:
        """绘制分布图"""
        try:
            fig = self._create_figure()
            if pd.api.types.is_numeric_dtype(data):
                sns.histplot(data, kde=True)
                plt.title(f"{column} 的分布", fontproperties=self.font_prop)
                plt.xlabel(column, fontproperties=self.font_prop)
                plt.ylabel('频数', fontproperties=self.font_prop)
            else:
                sns.countplot(x=data)
                plt.title(f"{column} 的频数分布", fontproperties=self.font_prop)
                plt.xlabel(column, fontproperties=self.font_prop)
                plt.ylabel('频数', fontproperties=self.font_prop)
                plt.xticks(rotation=45, fontproperties=self.font_prop)
                plt.yticks(fontproperties=self.font_prop)
            
            buf = BytesIO()
            plt.savefig(buf, format='png', bbox_inches='tight', dpi=100)
            buf.seek(0)
            img_base64 = base64.b64encode(buf.read()).decode('utf-8')
            self._close_figure(fig)
            return img_base64
        except Exception as e:
            logger.error(f"绘制分布图时出错: {str(e)}")
            self._close_figure(fig)
            raise
    
    def plot_correlation_heatmap(self, corr_matrix: pd.DataFrame) -> str:
        """绘制相关性热图"""
        try:
            fig = self._create_figure((12, 8))
            sns.heatmap(corr_matrix, annot=True, cmap='coolwarm', fmt='.2f')
            plt.title('相关性热图', fontproperties=self.font_prop)
            plt.xticks(fontproperties=self.font_prop)
            plt.yticks(fontproperties=self.font_prop)
            
            buf = BytesIO()
            plt.savefig(buf, format='png', bbox_inches='tight', dpi=100)
            buf.seek(0)
            img_base64 = base64.b64encode(buf.read()).decode('utf-8')
            self._close_figure(fig)
            return img_base64
        except Exception as e:
            logger.error(f"绘制相关性热图时出错: {str(e)}")
            self._close_figure(fig)
            raise
    
    def clear_cache(self) -> None:
        """清除图表缓存"""
        for fig in self._figure_cache.values():
            self._close_figure(fig)
        self._figure_cache.clear()

# 工具函数
def load_data(file_path: str) -> pd.DataFrame:
    """加载数据文件"""
    # 根据文件类型加载数据
    if file_path.endswith('.csv'):
        df = pd.read_csv(file_path)
    elif file_path.endswith(('.xls', '.xlsx')):
        df = pd.read_excel(file_path)
    elif file_path.endswith('.json'):
        df = pd.read_json(file_path)
    else:
        raise ValueError("不支持的文件格式")
    
    return df

def analyze_data(file_path: str, output_html: str = "analysis_report.html") -> None:
    """分析数据并生成报告"""
    df = load_data(file_path)
    analyzer = DataAnalyzer(df)
    analyzer.generate_report(output_html)

def analyze_multiple_files(folder_path: str, output_dir: str = "reports") -> None:
    """分析文件夹中的所有数据文件"""
    os.makedirs(output_dir, exist_ok=True)
    
    for file in os.listdir(folder_path):
        if file.endswith(('.csv', '.xls', '.xlsx', '.json')):
            file_path = os.path.join(folder_path, file)
            output_html = os.path.join(output_dir, f"{os.path.splitext(file)[0]}_report.html")
            try:
                analyze_data(file_path, output_html)
                print(f"已分析文件: {file}")
            except Exception as e:
                print(f"分析文件 {file} 时出错: {str(e)}")

def analyze_notebook(df: pd.DataFrame) -> None:
    """在Jupyter Notebook中分析数据并显示报告
    
    参数:
        df: pandas DataFrame，要分析的数据集
    """
    try:
        analyzer = DataAnalyzer(df)
        analyzer.analyze_notebook()
    except Exception as e:
        print(f"分析数据时出错: {str(e)}")

class ReportGenerator:
    """报告生成工具类"""
    
    def __init__(self, template_dir: str, static_dir: str):
        self.template_dir = template_dir
        self.static_dir = static_dir
        self.env = Environment(loader=FileSystemLoader(template_dir))
    
    def _copy_static_files(self) -> None:
        """复制静态文件到输出目录"""
        import shutil
        
        # 源文件路径
        source_js = os.path.join(os.path.dirname(__file__), 'static', 'js', 'analyzer.js')
        target_js = os.path.join(self.static_dir, 'js', 'analyzer.js')
        
        # 确保目标目录存在
        os.makedirs(os.path.dirname(target_js), exist_ok=True)
        
        # 复制文件
        try:
            if os.path.exists(source_js):
                if os.path.exists(target_js):
                    # 如果目标文件存在且与源文件不同，则更新
                    if not self._files_are_identical(source_js, target_js):
                        shutil.copy2(source_js, target_js)
                        logger.info(f"更新了JavaScript文件: {target_js}")
                else:
                    shutil.copy2(source_js, target_js)
                    logger.info(f"复制了JavaScript文件: {target_js}")
            else:
                logger.warning(f"警告: 源文件不存在: {source_js}")
        except Exception as e:
            logger.error(f"复制静态文件时出错: {str(e)}")
    
    @staticmethod
    def _files_are_identical(file1: str, file2: str) -> bool:
        """比较两个文件是否相同"""
        try:
            with open(file1, 'rb') as f1, open(file2, 'rb') as f2:
                return f1.read() == f2.read()
        except Exception:
            return False
    
    def generate_report(self, analysis_results: Dict[str, Any], output_html: str) -> None:
        """生成分析报告"""
        try:
            logger.info("开始生成报告...")
            
            # 确保静态文件目录存在并复制静态文件
            os.makedirs(self.static_dir, exist_ok=True)
            os.makedirs(os.path.join(self.static_dir, 'js'), exist_ok=True)
            self._copy_static_files()
            
            # 加载模板
            template = self.env.get_template('analyzer.html')
            logger.info("模板加载成功")
            
            # 获取输出文件的目录
            output_dir = os.path.dirname(os.path.abspath(output_html))
            if not output_dir:
                output_dir = os.getcwd()
            logger.info(f"输出目录: {output_dir}")
            
            # 计算静态文件的相对路径
            static_url = os.path.relpath(self.static_dir, output_dir)
            if static_url.startswith('..'):
                static_url = os.path.join('..', static_url)
            logger.info(f"静态文件URL: {static_url}")
            
            # 处理数据并转换为JSON
            processed_data = DataProcessor.process_data(analysis_results)
            json_data = json.dumps(processed_data, ensure_ascii=False)
            
            # 渲染模板
            logger.info("开始渲染模板...")
            rendered_html = template.render(
                data=json_data,
                timestamp=datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                static_url=static_url
            )
            logger.info("模板渲染完成")
            
            # 保存报告
            logger.info(f"保存报告到: {output_html}")
            with open(output_html, 'w', encoding='utf-8') as f:
                f.write(rendered_html)
            logger.info(f"分析报告已保存至: {output_html}")
            
        except Exception as e:
            logger.error(f"生成报告时出错: {str(e)}")
            raise



# 放置一个经常用到的函数

def pypis():
    # 常用的 pypi 更新源地址
    print("在国内使用 pip 安装 Python 包时，经常会遇到下载速度慢的问题。")
    print("这是因为 pip 默认从国外的 PyPI 服务器下载包，速度比较慢。")
    print("可以通过更换国内的源地址来解决这个问题。")
    print("以下是一些常用的国内源地址：")
    print("阿里云 https://mirrors.aliyun.com/pypi/simple/")
    print("清华大学 https://pypi.tuna.tsinghua.edu.cn/simple/")
    print("中国科学技术大学 https://pypi.mirrors.ustc.edu.cn/simple/")
    print("豆瓣 https://pypi.doubanio.com/simple/")
    print("网易 https://mirrors.163.com/pypi/simple/")
    print("腾讯云 https://mirrors.cloud.tencent.com/pypi/simple")
    print("\n","-"*40)
    print("""
使用说明

一、临时使用
使用pip的时候在后面加上-i参数，指定pip源：

pip install xxx -i https://mirrors.163.com/pypi/simple/
替换“xxx”为你需要安装的模块名称。

二、永久修改使用
Linux/Unix中使用：

~/.pip/pip.conf
添加或修改pip.conf（如果不存在，创建一个）

[global]
index-url = https://mirrors.163.com/pypi/simple/

Windows中使用：

%APPDATA%/pip/pip.ini
1.打开此电脑，在最上面的的文件夹窗口输入：%APPDATA%

2.按回车跳转进入目录，并新建一个文件夹：pip

3.创建文件：pip.ini

添加或修改pip.ini（如果不存在，创建一个）

[global]
index-url = https://mirrors.163.com/pypi/simple/
    """)

def dep():
    print("获取项目依赖库的版本信息")
    print("使用方法：")
    print("https://github.com/WangLaoShi/pipdeptree")
# pypis()

def styles():
    print("*"*60)
    print("JupyterNotebook 支持的 Markdown 样式")
    print("标题")
    print("# 一级标题")
    print("## 二级标题")
    print("### 三级标题")
    print("#### 四级标题")
    print("##### 五级标题")
    print("###### 六级标题")
    print("\n")
    print("列表")
    print("- 无序列表")
    print("1. 有序列表")
    print("\n")
    print("链接")
    print("[链接名称](链接地址)")
    print("\n")
    print("图片")
    print("![图片名称](图片地址)")
    print("\n")
    print("引用")
    print("> 引用内容")
    print("\n")
    print("代码")
    print("```python")
    print("print('Hello World!')")
    print("```")
    print("\n")
    print("表格")
    print("| 表头1 | 表头2 |")
    print("| --- | --- |")
    print("| 内容1 | 内容2 |")
    print("\n")
    print("加粗")
    print("**加粗内容**")
    print("\n")
    print("斜体")
    print("*斜体内容*")
    print("\n")
    print("删除线")
    print("~~删除线内容~~")
    print("\n")
    print("分割线")
    print("---")
    print("\n")
    print("脚注")
    print("脚注[^1]")
    print("[^1]: 脚注内容")
    print("\n")
    print("格式化-背景颜色")
    print("# <div style='background-color:skyblue'><center> TEXT WITH BACKGROUND COLOR </center></div>")
    print("\n")
    print("格式化-背景颜色")
    print("""
# Blue Background
<div class="alert alert-info"> Example text highlighted in blue background </div>
# Green Background
<div class="alert alert-success">Example text highlighted in green background.</div>
# Yellow Background
<div class="alert alert-warning">Example text highlighted in yellow background.</div>
# Red Background
<div class="alert alert-danger">Example text highlighted in red background.</div>
    """)
    print("\n")

    print("格式化-符号")
    print("""
&#10148; Bullet point one</br>
&#10143; Bullet point two</br>
&#10147; Bullet point three</br>
&#10145; Bullet point four</br>
&#10144; Bullet point five</br>
&#10142; Bullet point six</br>
&#10141; Bullet point seven</br>
&#10140; Bullet point eight</br>
    """)
    print("\n")
    print("格式化-有色文本")
    print("""
print('\033[31;3m This is red\033[0m')
print('\033[32;3m This is green\033[0m')
print('\033[33;3m This is yellow\033[0m')
print('\033[34;3m This is blue\033[0m')
print('\033[35;3m This is pink\033[0m')
print('\033[36;3m This is skyblue\033[0m')
print('\033[37;3m This is grey\033[0m')
    """)
    print("\n")
    print("格式化-黑体文字")
    print("""
print('\033[1;31m This is bold red \033[0m')
print('\033[1;32m This is bold green\033[0m')
print('\033[1;33m This is bold yellow\033[0m')
print('\033[1;34m This is bold blue\033[0m')
print('\033[1;35m This is bold purple\033[0m')
print('\033[1;36m This is bold teal\033[0m')
print('\033[1;37m This is bold grey\033[0m')
    """)
    print("\n")
    print("格式化-背景颜色")
    print("""
print('\033[1;40mBlack background - Bold text\033[0m')
print('\033[1;41mRed background - Bold text\033[0m')
print('\033[1;42mGreen background - Bold text\033[0m')
print('\033[1;43mYellow background - Bold text\033[0m')
print('\033[1;44mBlue background - Bold text\033[0m')
print('\033[1;45mPink background - Bold text\033[0m')
print('\033[1;46mLight Blue background - Bold text\033[0m')
print('\033[1;47mLight Grey background - Bold text\033[0m')    
    """)
    print("*"*60)
    print("\n")
    print("*"*60)

def helps():
    print("本模块包含一些常用的函数，可以直接调用。")
    print("pypis()  # 常用的 pypi 更新源地址")
    print("dep()  # 获取项目依赖库的版本信息")
    print("helps()  # 查看帮助信息")
    print("styles()  # 查看 JupyterNotebook 支持的样式")
    
styles()


"""
机器学习和深度学习基础方法模块

本模块提供了常用的机器学习和深度学习方法的实现，包括：
1. 数据预处理
2. 特征工程
3. 基础机器学习模型
4. 深度学习模型
5. 模型评估
6. 可视化工具
"""

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from sklearn.preprocessing import StandardScaler, MinMaxScaler, LabelEncoder
from sklearn.model_selection import train_test_split, cross_val_score, GridSearchCV
from sklearn.metrics import accuracy_score, precision_score, recall_score, f1_score
from sklearn.metrics import mean_squared_error, r2_score, confusion_matrix
from sklearn.linear_model import LinearRegression, LogisticRegression
from sklearn.tree import DecisionTreeClassifier, DecisionTreeRegressor
from sklearn.ensemble import RandomForestClassifier, RandomForestRegressor
from sklearn.svm import SVC, SVR
from sklearn.neighbors import KNeighborsClassifier, KNeighborsRegressor
import torch
import torch.nn as nn
import torch.optim as optim
from torch.utils.data import Dataset, DataLoader

class DataPreprocessor:
    """数据预处理类
    
    提供数据清洗、标准化、编码等基础预处理功能
    """
    
    def __init__(self):
        """初始化预处理器"""
        self.scaler = StandardScaler()
        self.label_encoder = LabelEncoder()
        
    def handle_missing_values(self, df: pd.DataFrame, method: str = 'mean') -> pd.DataFrame:
        """处理缺失值
        
        参数:
            df: 输入数据框
            method: 填充方法，可选 'mean', 'median', 'mode', 'drop'
            
        返回:
            处理后的数据框
        """
        df_processed = df.copy()
        
        if method == 'mean':
            df_processed = df_processed.fillna(df_processed.mean())
        elif method == 'median':
            df_processed = df_processed.fillna(df_processed.median())
        elif method == 'mode':
            df_processed = df_processed.fillna(df_processed.mode().iloc[0])
        elif method == 'drop':
            df_processed = df_processed.dropna()
            
        return df_processed
    
    def encode_categorical(self, df: pd.DataFrame, columns: list) -> pd.DataFrame:
        """对分类变量进行编码
        
        参数:
            df: 输入数据框
            columns: 需要编码的列名列表
            
        返回:
            编码后的数据框
        """
        df_encoded = df.copy()
        for col in columns:
            df_encoded[col] = self.label_encoder.fit_transform(df_encoded[col])
        return df_encoded
    
    def scale_features(self, df: pd.DataFrame, columns: list) -> pd.DataFrame:
        """特征标准化
        
        参数:
            df: 输入数据框
            columns: 需要标准化的列名列表
            
        返回:
            标准化后的数据框
        """
        df_scaled = df.copy()
        df_scaled[columns] = self.scaler.fit_transform(df_scaled[columns])
        return df_scaled

class FeatureEngineer:
    """特征工程类
    
    提供特征选择、特征创建、特征转换等功能
    """
    
    def __init__(self):
        """初始化特征工程器"""
        pass
        
    def create_polynomial_features(self, df: pd.DataFrame, columns: list, degree: int = 2) -> pd.DataFrame:
        """创建多项式特征
        
        参数:
            df: 输入数据框
            columns: 需要创建多项式特征的列名列表
            degree: 多项式次数
            
        返回:
            添加了多项式特征的数据框
        """
        df_poly = df.copy()
        for col in columns:
            for d in range(2, degree + 1):
                df_poly[f'{col}_power_{d}'] = df_poly[col] ** d
        return df_poly
    
    def create_interaction_features(self, df: pd.DataFrame, columns: list) -> pd.DataFrame:
        """创建交互特征
        
        参数:
            df: 输入数据框
            columns: 需要创建交互特征的列名列表
            
        返回:
            添加了交互特征的数据框
        """
        df_inter = df.copy()
        for i in range(len(columns)):
            for j in range(i + 1, len(columns)):
                col1, col2 = columns[i], columns[j]
                df_inter[f'{col1}_{col2}_interaction'] = df_inter[col1] * df_inter[col2]
        return df_inter

class MLModel:
    """机器学习模型类
    
    提供常用机器学习模型的训练、预测和评估功能
    """
    
    def __init__(self, model_type: str):
        """初始化模型
        
        参数:
            model_type: 模型类型，可选 'linear', 'logistic', 'tree', 'forest', 'svm', 'knn'
        """
        self.model_type = model_type
        self.model = self._create_model()
        
    def _create_model(self):
        """创建模型实例"""
        if self.model_type == 'linear':
            return LinearRegression()
        elif self.model_type == 'logistic':
            return LogisticRegression()
        elif self.model_type == 'tree':
            return DecisionTreeClassifier()
        elif self.model_type == 'forest':
            return RandomForestClassifier()
        elif self.model_type == 'svm':
            return SVC()
        elif self.model_type == 'knn':
            return KNeighborsClassifier()
        else:
            raise ValueError(f"不支持的模型类型: {self.model_type}")
    
    def train(self, X: pd.DataFrame, y: pd.Series, test_size: float = 0.2):
        """训练模型
        
        参数:
            X: 特征数据
            y: 目标变量
            test_size: 测试集比例
        """
        self.X_train, self.X_test, self.y_train, self.y_test = train_test_split(
            X, y, test_size=test_size, random_state=42
        )
        self.model.fit(self.X_train, self.y_train)
        
    def predict(self, X: pd.DataFrame) -> np.ndarray:
        """预测
        
        参数:
            X: 特征数据
            
        返回:
            预测结果
        """
        return self.model.predict(X)
    
    def evaluate(self) -> dict:
        """评估模型
        
        返回:
            包含各项评估指标的字典
        """
        y_pred = self.predict(self.X_test)
        
        if self.model_type in ['linear']:
            metrics = {
                'MSE': mean_squared_error(self.y_test, y_pred),
                'R2': r2_score(self.y_test, y_pred)
            }
        else:
            metrics = {
                'Accuracy': accuracy_score(self.y_test, y_pred),
                'Precision': precision_score(self.y_test, y_pred, average='weighted'),
                'Recall': recall_score(self.y_test, y_pred, average='weighted'),
                'F1': f1_score(self.y_test, y_pred, average='weighted')
            }
            
        return metrics

class NeuralNetwork(nn.Module):
    """神经网络模型类
    
    提供基础神经网络模型的实现
    """
    
    def __init__(self, input_size: int, hidden_size: int, output_size: int):
        """初始化神经网络
        
        参数:
            input_size: 输入层大小
            hidden_size: 隐藏层大小
            output_size: 输出层大小
        """
        super(NeuralNetwork, self).__init__()
        self.layer1 = nn.Linear(input_size, hidden_size)
        self.relu = nn.ReLU()
        self.layer2 = nn.Linear(hidden_size, output_size)
        
    def forward(self, x):
        """前向传播
        
        参数:
            x: 输入数据
            
        返回:
            模型输出
        """
        x = self.layer1(x)
        x = self.relu(x)
        x = self.layer2(x)
        return x

class DLModel:
    """深度学习模型类
    
    提供深度学习模型的训练、预测和评估功能
    """
    
    def __init__(self, input_size: int, hidden_size: int, output_size: int):
        """初始化深度学习模型
        
        参数:
            input_size: 输入层大小
            hidden_size: 隐藏层大小
            output_size: 输出层大小
        """
        self.model = NeuralNetwork(input_size, hidden_size, output_size)
        self.criterion = nn.CrossEntropyLoss()
        self.optimizer = optim.Adam(self.model.parameters())
        
    def train(self, X: torch.Tensor, y: torch.Tensor, epochs: int = 100):
        """训练模型
        
        参数:
            X: 特征数据
            y: 目标变量
            epochs: 训练轮数
        """
        for epoch in range(epochs):
            # 前向传播
            outputs = self.model(X)
            loss = self.criterion(outputs, y)
            
            # 反向传播和优化
            self.optimizer.zero_grad()
            loss.backward()
            self.optimizer.step()
            
            if (epoch + 1) % 10 == 0:
                print(f'Epoch [{epoch+1}/{epochs}], Loss: {loss.item():.4f}')
                
    def predict(self, X: torch.Tensor) -> torch.Tensor:
        """预测
        
        参数:
            X: 特征数据
            
        返回:
            预测结果
        """
        with torch.no_grad():
            outputs = self.model(X)
            _, predicted = torch.max(outputs.data, 1)
        return predicted

class ModelEvaluator:
    """模型评估类
    
    提供模型评估和可视化功能
    """
    
    def __init__(self):
        """初始化评估器"""
        pass
        
    def plot_confusion_matrix(self, y_true: np.ndarray, y_pred: np.ndarray):
        """绘制混淆矩阵
        
        参数:
            y_true: 真实标签
            y_pred: 预测标签
        """
        cm = confusion_matrix(y_true, y_pred)
        plt.figure(figsize=(8, 6))
        sns.heatmap(cm, annot=True, fmt='d', cmap='Blues')
        plt.title('混淆矩阵')
        plt.ylabel('真实标签')
        plt.xlabel('预测标签')
        plt.show()
        
    def plot_learning_curve(self, train_sizes: np.ndarray, train_scores: np.ndarray, 
                          test_scores: np.ndarray):
        """绘制学习曲线
        
        参数:
            train_sizes: 训练集大小
            train_scores: 训练集得分
            test_scores: 测试集得分
        """
        plt.figure(figsize=(8, 6))
        plt.plot(train_sizes, np.mean(train_scores, axis=1), label='训练集得分')
        plt.plot(train_sizes, np.mean(test_scores, axis=1), label='测试集得分')
        plt.xlabel('训练样本数')
        plt.ylabel('得分')
        plt.title('学习曲线')
        plt.legend()
        plt.show()

# 使用示例
def example_usage():
    """使用示例"""
    # 1. 数据预处理
    preprocessor = DataPreprocessor()
    df = pd.DataFrame({
        'A': [1, 2, np.nan, 4, 5],
        'B': ['a', 'b', 'a', 'c', 'b']
    })
    df_processed = preprocessor.handle_missing_values(df, method='mean')
    df_encoded = preprocessor.encode_categorical(df_processed, ['B'])
    
    # 2. 特征工程
    engineer = FeatureEngineer()
    df_features = engineer.create_polynomial_features(df_encoded, ['A'], degree=2)
    
    # 3. 机器学习模型
    ml_model = MLModel('logistic')
    X = df_features[['A', 'A_power_2']]
    y = df_features['B']
    ml_model.train(X, y)
    metrics = ml_model.evaluate()
    print("ML模型评估结果:", metrics)
    
    # 4. 深度学习模型
    dl_model = DLModel(input_size=2, hidden_size=4, output_size=3)
    X_tensor = torch.FloatTensor(X.values)
    y_tensor = torch.LongTensor(y.values)
    dl_model.train(X_tensor, y_tensor, epochs=100)
    
    # 5. 模型评估
    evaluator = ModelEvaluator()
    y_pred = ml_model.predict(X)
    evaluator.plot_confusion_matrix(y, y_pred)

if __name__ == '__main__':
    example_usage()



from setuptools import setup, find_packages, Command
import os
import sys

about = {}
about['__version__'] = '0.10.11'
about['__project_name__'] = 'wanglaoshi'

class UploadCommand(Command):
    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        self.status('Clean build,dist,wanglaoshi.egg-info directory …')
        os.system('rm -rf ./build ./dist ./*.egg-info')
        self.status('Building Source and Wheel distribution…')
        os.system('{0} setup.py sdist bdist_wheel'.format(sys.executable))

        self.status('Uploading the package to PyPI via Twine…')
        os.system('twine upload dist/*')

        self.status('Pushing git tags…')
        os.system('git tag v{0}'.format(about['__version__']))
        os.system('git add .')
        os.system('git commit -m v{0} .'.format(about['__version__']))
        os.system('git push --tags')
        os.system('git push')
        sys.exit()

setup(
    name=about['__project_name__'],  # 包的名称
    version=about['__version__'],  # 版本号
    packages=find_packages(),  # 自动找到所有模块
    install_requires=[         # 依赖的库
        'rich',
        'requests',
        'pandas',
        'numpy',
        'matplotlib',
        'seaborn',
        'scipy',
        'missingno',
        'jinja2',
        'tqdm',
        'statsmodels',
        'scikit-learn',
        # 在这里列出其他依赖的库
    ],
    author='WangLaoShi',  # 作者
    author_email='ginger547@gmail.com',  # 邮箱
    description='A utility module for DA and ML and DL tasks',  # 简短描述
    long_description=open('README.md').read(),  # 从 README.md 文件读取详细描述
    long_description_content_type='text/markdown',  # README.md 文件格式
    url='https://github.com/wanglaoshi/wanglaoshi-pypi',  # 项目的 GitHub 链接
    classifiers=[   # 分类信息
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.8',  # Python 版本要求
    cmdclass={
        'upload': UploadCommand,
    }
)